package com.persagy.dc.define.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.persagy.common.exception.BusinessException;
import com.persagy.dc.common.constant.ResponseCode;
import com.persagy.dc.basic.utils.DigitalCommonUtils;
import com.persagy.dc.basic.model.QueryCriteria;
import com.persagy.dc.common.model.response.CommonResult;
import com.persagy.dc.common.model.response.PageList;
import com.persagy.dc.common.utils.ResultHelper;
import com.persagy.dc.define.entity.ObjectInfo;
import com.persagy.dc.define.service.IObjectInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 对象类型 Controller
 * @author Charlie Yu
 * @date 2021-06-25
 */
@RestController
@RequestMapping("/rwd/def/class")
public class ObjectInfoController {

    @Autowired
    private IObjectInfoService service;

    @PostMapping("")
    public CommonResult<PageList<ObjectInfo>> queryClass(@RequestBody QueryCriteria criteria) {
        if(criteria == null) {
            throw new BusinessException(ResponseCode.A0400.getCode(), ResponseCode.A0400.getDesc());
        }
        QueryWrapper<ObjectInfo> wrapper = new QueryWrapper<>();
        // 添加所属项目条件
        DigitalCommonUtils.ensureProjectCriteria(wrapper);
        // 转换查询条件
        criteria.toWrapper(wrapper);
        Page page = service.queryByCondition(criteria.toPage(), wrapper);
        return ResultHelper.multi(page.getRecords());
    }

    @PostMapping("/create")
    public CommonResult<ObjectInfo> create(@RequestBody ObjectInfo vo){
        vo = service.insert(vo);
        return ResultHelper.single(vo);
    }

    @PostMapping("/update")
    public CommonResult<ObjectInfo> update(@RequestBody ObjectInfo vo){
        vo = service.update(vo);
        return ResultHelper.single(vo);
    }

    @PostMapping("/delete")
    public void delete(@RequestBody String id){
        service.delete(id);
    }
}
